import { AppStylesTargetBehavior } from './behaviors';
import { AppConfig } from './config';
import type { AppAssets, AppElement, AppMetadata, AppRegister, AppRegistration, AppRoute, AppStore, AppStyle, AppValidity } from './types';
/**
 * `App` provides APIs to register integration targets for PBCs and other actors to register their assets in bulk via
 * token exports. It also provides specific APIs to add single assets to the application itself, like routes, elements,
 * styles, store fragments etc.
 *
 * @example
 * App reference via DI.
 * ```ts
 * import { App } from '@genesislcap/foundation-shell/app';
 * ...
 * @App private app: App;
 * ```
 *
 * @example
 * App reference via getApp util.
 * ```ts
 * import { getApp } from '@genesislcap/foundation-shell/app';
 * ...
 * const app = getApp();
 * ```
 *
 * @public
 */
export interface App extends AppAssets, AppRegister, AppValidity {
    /**
     * High-level app configuration.
     */
    readonly config: AppConfig;
    /**
     * Register assets from either a globed dynamic import or some predefined AppAssets instance. This is a one time
     * operation. Assets had be subsequently registered and deregistered using their individual APIs.
     *
     * @example
     * Manual `AppAssets` registration.
     * ```ts
     * app.registerAssets({
     *   // with or without pbcAssets
     *   elements: [...],
     *   metadata: [...],
     *   registrations: [...],
     *   routes: [...],
     *   styles: [...],
     *   stores: [...],
     * });
     * ```
     *
     * @example
     * PBC `AppAssets` registration.
     * ```ts
     * import { importPBCAssets } from '@genesislcap/foundation-shell/pbc';
     * ...
     * const pbcAssets = await importPBCAssets();
     * ...
     * app.registerAssets(pbcAssets);
     * ```
     *
     * @param assets - {@link AppAssets}.
     */
    registerAssets(assets: Partial<AppAssets>): void;
    /**
     * Deregister assets.
     */
    deregisterAssets(): void;
    /**
     * Check if the App has registered assets.
     */
    hasAssets(): boolean;
}
/**
 * Default App implementation.
 * @public
 */
export declare class DefaultApp implements App {
    /**
     * {@inheritDoc App.config}
     * @internal
     */
    config: AppConfig;
    /** {@inheritDoc AppAssets.elements} */
    elements: AppElement[];
    /** {@inheritDoc AppAssets.metadata} */
    metadata: AppMetadata[];
    /** {@inheritDoc AppAssets.registrations} */
    registrations: AppRegistration[];
    /** {@inheritDoc AppAssets.routes} */
    routes: AppRoute[];
    /** {@inheritDoc AppAssets.styles} */
    styles: AppStyle[];
    /** {@inheritDoc AppAssets.stores} */
    stores: AppStore[];
    /** {@inheritDoc AppRegister.registerElementsTarget} */
    registerElementsTarget(targetId: any, predicate: any, repeatOptions?: import("@microsoft/fast-element").RepeatOptions): import("@microsoft/fast-element").ViewTemplate<any, any>;
    /** {@inheritDoc AppRegister.registerElement} */
    registerElement(element: any): void;
    /** {@inheritDoc AppRegister.deregisterElement} */
    deregisterElement(element: any): void;
    /** {@inheritDoc AppRegister.registerRouteCollection} */
    registerRouteCollection(routeCollection: any, predicate: any): any;
    /** {@inheritDoc AppRegister.registerRoute} */
    registerRoute(route: any): void;
    /** {@inheritDoc AppRegister.deregisterRoute} */
    deregisterRoute(route: any): void;
    /** {@inheritDoc AppRegister.registerStoreRoot} */
    registerStoreRoot(storeRoot: any, predicate: any): () => void;
    /** {@inheritDoc AppRegister.registerStore} */
    registerStore(store: any): void;
    /** {@inheritDoc AppRegister.deregisterStore} */
    deregisterStore(store: any): void;
    /** {@inheritDoc AppRegister.registerStylesTarget} */
    registerStylesTarget(targetId: any, predicate: any): AppStylesTargetBehavior;
    /** {@inheritDoc AppRegister.registerStyle} */
    registerStyle(style: any): void;
    /** {@inheritDoc AppRegister.deregisterStyle} */
    deregisterStyle(style: any): void;
    /** {@inheritDoc AppRegister.registerComponents} */
    registerComponents(context: any): void;
    /**
     * {@inheritDoc AppValidity.isValid}
     *
     * @privateRemarks
     * This is a placeholder to potentially monitor metadata etc. as assets are registered and deregistered.
     */
    isValid(context: any): boolean;
    /** {@inheritDoc App.registerAssets} */
    registerAssets(assets: any): void;
    /** {@inheritDoc App.deregisterAssets} */
    deregisterAssets(): void;
    /** {@inheritDoc App.hasAssets} */
    hasAssets(): boolean;
}
/**
 * App DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const App: import("@microsoft/fast-foundation").InterfaceSymbol<App>;
/**
 * Gets App from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getApp } from '@genesislcap/foundation-shell/app';
 * ...
 * private user = getApp();
 * ```
 *
 * @public
 */
export declare function getApp(): App;
//# sourceMappingURL=app.d.ts.map