import { AppAssets } from '../app';
/**
 * A utility for importing PBC assets that have been added to the application's src/pbc directory via genx add.
 *
 * The `genx` builder (webpack or vite) will attempt to dynamically load all PBC assets from the application's `src/pbc`
 * directory by default. The returned AppAssets can then be applied to the App. Please note that `app.registerAssets`
 * is a one-time operation, intended for upfront bulk registration. After this point, you must use the single asset
 * APIs of the `App` for lazy asset registration or deregistration.
 *
 * @example
 * ```ts
 * import { importPBCAssets } from '@genesislcap/foundation-shell/pbc';
 * ...
 * const assets = await importPBCAssets();
 * ...
 * app.registerAssets(assets);
 * ```
 *
 * @example
 * A typical app setup that makes use of PBC imports might look like this.
 * ```ts
 * // src/main/main.ts
 *
 * @App app: App;
 *
 * async connectedCallback() {
 *   ...
 *   await this.loadPBCs();
 *   ...
 * }
 *
 * async loadPBCs() {
 *   // Import PBC assets that may have been added to the ./pbc directory by genx or by hand
 *   const pbcAssets = await importPBCAssets();
 *
 *   // Register bulk assets
 *   this.app.registerAssets(pbcAssets);
 *
 *   // Register the top-level route collection
 *   this.app.registerRouteCollection(this.config.routes);
 * }
 * ```
 *
 * The rest of the PBC setup is handled by the system automatically, however, you will still need to register
 * application asset integration points.
 *
 * @example
 * A typical PBC directory structure may look like this.
 * ```markdown
 * ├── reporting
 * │   ├── elements.ts
 * │   ├── metadata.ts
 * │   ├── registrations.ts
 * │   ├── routes.ts
 * │   ├── stores.ts
 * │   └── styles.ts
 * ```
 *
 * Each file is dynamically loaded and processed as part of the overall PBC set the application contains. PBCs do not
 * need to include `index.ts` files, and export token naming does not matter within any of these files. For example,
 * `foo` here `export const foo: AppElement = {...}` has no impact on PBC integration.
 *
 * The answers generated from the `genx add` cli flows can be used to tailor what PBC assets are included at build-time.
 *
 * @public
 */
export declare function importPBCAssets(): Promise<AppAssets>;
//# sourceMappingURL=pbc.d.ts.map